//Attention, les lois géométriques de Scilab ne sont pas les mêmes que celles de la feuille de TP.
//On peut en effet considérer le nombre d'essais pour avoir un succès (Scilab) ou le nombre d'échecs (TP).
//Faire l'essai pour p=1
function Z=traj(p,n)
    Z=zeros(n+1,1);
    Z(1)=1;
    for i=1:n
        if Z(i)==0 then
            break
        end
        Z(i+1)=sum(grand(Z(i),1,'geom',p)-1)
    end
endfunction
//Cette fonction génère une réalisation d'un arbre de Galton Watson. Il n'y a plus qu'à tracer.


n=5;plot2d(0:n,traj(0.3,n),5)
n=20;plot2d(0:n,traj(2,8,n),5)
//On peut faire quelques vérifications de la validité du programme.


//Utilisons la méthode de Monte-Carlo pour estimer l'espérance à l'instant n sur un échantillon de plusieurs trajectoires
p=0.4;n=5;m=1/p-1;
X=zeros(10000,1);
for i=1:length(X)
    Z=traj(p,n);X(i)=Z(length(Z));
end
abs(mean(X)-m^n)



p=0.25;n=10;
//T étant infini avec proba non-nulle, il est nécessaire d'arrêter la chaine au bout d'un nombre fini d'étapes, ou si elle dépasse un certain niveau (la mémoire disponible dans Scilab étant peu élevée, on pourra choisir la première solution avec n pas trop élevé)
P=zeros(10000,1);
for i=1:length(P)
    Z=traj(p,n);P(i)=bool2s(Z(length(Z))==0);
end
abs(mean(P)-p/(1-p))



//Voyons d'abord le cas m=1
n=80;
clear P;
for k=1:n
    clear Z;
    for i=1:2000
        Z(:,i)=traj(0.5,k);
    end
    clear X;
    X=Z(k+1,:);X=bool2s(X>0);
    P(k)=mean(X)*k;
end
plot(P)



n=50;x=[0:0.1:20]
clear P;
for k=1:n
    clear Z;
    for i=1:4000
        Z(:,i)=traj(0.5,k);
    end
    clear X;
    X=Z(k+1,:);X=X(X>0)/k;
    clf
    histplot(floor(sqrt(length(X))),X);plot(x,exp(-x))
    //On peut voir la "convergence de l'istogramme" vers la densité de Exp(1)
end



//Voyons maintenant le cas m>1
n=20;p=0.4;m=1/p-1;
Z=traj(p,n);
for k=1:n
    Z(k+1)=Z(k+1)/m^k;
end
plot([0:n],Z)
//Faire tourner ces lignes de commande quelques fois suffisent à se convaincre qu'on n'a pas convergence presque sûre. Voyons la convergence en loi.



n=30;p=0.45;m=1/p-1;sigma=(1-p)/(p^2);clear Z
for i=1:3000
    Z(:,i)=traj(p,n);
end
for k=1:n
    Z(k+1,:)=Z(k+1,:)/m^k;
end
Y=Z(n+1,:);
histplot(floor(sqrt(length(Y))),Y)
//L'histogramme a l'air d'être toujours le même, et l'on peut tester les choses suivantes :
abs(1-mean(Y))
//abs(n*variance(Y)/(n-1)-sigma/(m^2-m))
sigma/(m^2-m)
















